==> la_panic-0.4.3.tar.gz: la_panic-0.4.3/la_panic/panic_parser/panic_specific/kernel_specific_base.py <==
from abc import ABC, abstractmethod
from typing import Optional, List

from la_panic.data_structure.raw_crash_stack import RawCrashStack
from la_panic.panic_parser.backtrace import Backtrace
from la_panic.panic_parser.core_state import CoreState, CrashedCoreState, CoreInfo
from la_panic.panic_parser.kext import LoadedKextInfo
from la_panic.panic_parser.sliders import KernelSliders
from la_panic.panic_parser.versions import KernelPanicVersions
from la_panic.panic_parser.zone import ZoneInfo


class KernelSpecificPanic(ABC):

    @property
    @abstractmethod
    def backtrace(self) -> Backtrace:
        pass

    @property
    @abstractmethod
    def kext_info(self) -> LoadedKextInfo:
        pass

    @staticmethod
    @abstractmethod
    def contains_crashed_core_info() -> bool:
        pass


class KernelSpecificBase(object):
    __general_info: str
    __release_type: str
    __versions: KernelPanicVersions
    __secure_boot: bool
    __roots_installed: int
    __panic_log_version: str
    __sliders: KernelSliders
    __zone_info: ZoneInfo
    __cores_count = 0
    __cores_state: [CoreState] = []
    __panicked_task: str

    def __init__(self, panic_infos: RawCrashStack, contains_crashed_core_info: bool):
        self.__general_info = panic_infos.pop_one()

        crashed_core_state = None
        if contains_crashed_core_info:
            crashed_core_state = CrashedCoreState(panic_infos.pop(9))

        # Remove Debugger message and Memory ID
        panic_infos.pop(2)

        self.__release_type = panic_infos.pop_value_from_key_value_pair()
        self.__versions = KernelPanicVersions(panic_infos)
        self.__secure_boot = True if panic_infos.pop_value_from_key_value_pair() == 'YES' else False
        self.__roots_installed = int(panic_infos.pop_value_from_key_value_pair())
        self.__panic_log_version = panic_infos.pop_value_from_key_value_pair()
        self.__sliders = KernelSliders(panic_infos)

        # Remove epoch time info
        panic_infos.pop(6)

        self.__zone_info = ZoneInfo(panic_infos)
        self.__cores_state = KernelSpecificBase.parse_cores_state(panic_infos, crashed_core_state)
        self.__compressor_info = panic_infos.pop_one()
        self.__panicked_task = panic_infos.pop_one()
        self.__panicked_thread = panic_infos.pop_one()

    @staticmethod
    def parse_cores_state(panic_infos: RawCrashStack, crashed_core_state: Optional[CrashedCoreState]) -> List[CoreInfo]:
        cores_retired_instruction_data = KernelSpecificBase.core_retired_instruction(panic_infos)
        cores_state = []

        core_states_raw = panic_infos.pop_until_line_containing('Compressor Info')

        for core_index, core_state_raw in enumerate(core_states_raw):
            try:
                core_retired_instruction = cores_retired_instruction_data[core_index]
            except IndexError:
                core_retired_instruction = None

            core_state = CoreInfo(core_index, core_retired_instruction, core_state_raw)
            cores_state.append(core_state)

            if core_state.crashed and crashed_core_state:
                core_state.update_crashed_state(crashed_core_state)

        return cores_state

    @staticmethod
    def core_retired_instruction(panic_infos: RawCrashStack) -> List[str]:
        cores_retired_instruction_data: List[str] = []

        while True:
            potential_retired_instruction_data = panic_infos.pop_one()
            if potential_retired_instruction_data.find("TPIDRx_ELy") != -1:
                break

            cores_retired_instruction_data.append(potential_retired_instruction_data)

        return cores_retired_instruction_data

    @property
    def crashed_core(self) -> CoreInfo:
        return list(filter(lambda core_state: core_state.crashed, self.__cores_state))[0]

    @property
    def sliders(self) -> KernelSliders:
        return self.__sliders

    @property
    def versions(self) -> KernelPanicVersions:
        return self.__versions

==> la_panic-0.4.5.tar.gz: la_panic-0.4.5/la_panic/panic_parser/panic_specific/kernel_specific_base.py <==
from abc import ABC, abstractmethod
from typing import Optional, List

from la_panic.data_structure.raw_crash_stack import RawCrashStack
from la_panic.panic_parser.backtrace import Backtrace
from la_panic.panic_parser.core_state import CoreState, CrashedCoreState, CoreInfo
from la_panic.panic_parser.kext import LoadedKextInfo
from la_panic.panic_parser.sliders import KernelSliders
from la_panic.panic_parser.versions import KernelPanicVersions
from la_panic.panic_parser.zone import ZoneInfo


class KernelSpecificPanic(ABC):

    @property
    @abstractmethod
    def backtrace(self) -> Backtrace:
        pass

    @property
    @abstractmethod
    def kext_info(self) -> LoadedKextInfo:
        pass

    @property
    @abstractmethod
    def crashed_core(self) -> CoreInfo:
        pass

    @property
    @abstractmethod
    def sliders(self) -> KernelSliders:
        pass

    @property
    @abstractmethod
    def versions(self) -> KernelPanicVersions:
        pass

    @staticmethod
    @abstractmethod
    def contains_crashed_core_info() -> bool:
        pass


class KernelSpecificBase(object):
    __general_info: str
    __release_type: str
    __versions: KernelPanicVersions
    __secure_boot: bool
    __roots_installed: int
    __panic_log_version: str
    __sliders: KernelSliders
    __zone_info: ZoneInfo
    __cores_count = 0
    __cores_state: [CoreState] = []
    __panicked_task: str

    def __init__(self, panic_infos: RawCrashStack, contains_crashed_core_info: bool):
        self.__general_info = panic_infos.pop_one()

        crashed_core_state = None
        if contains_crashed_core_info:
            crashed_core_state = CrashedCoreState(panic_infos.pop(9))

        # Remove Debugger message and Memory ID
        panic_infos.pop(2)

        self.__release_type = panic_infos.pop_value_from_key_value_pair()
        self.__versions = KernelPanicVersions(panic_infos)
        self.__secure_boot = True if panic_infos.pop_value_from_key_value_pair() == 'YES' else False
        self.__roots_installed = int(panic_infos.pop_value_from_key_value_pair())
        self.__panic_log_version = panic_infos.pop_value_from_key_value_pair()
        self.__sliders = KernelSliders(panic_infos)

        # Remove epoch time info
        panic_infos.pop(6)

        self.__zone_info = ZoneInfo(panic_infos)
        self.__cores_state = KernelSpecificBase.parse_cores_state(panic_infos,
                                                                  crashed_core_state, self.__sliders.kernel_cache_slide)
        self.__compressor_info = panic_infos.pop_one()
        self.__panicked_task = panic_infos.pop_one()
        self.__panicked_thread = panic_infos.pop_one()

    @staticmethod
    def parse_cores_state(panic_infos: RawCrashStack,
                          crashed_core_state: Optional[CrashedCoreState],
                          kernel_slide: hex) -> List[CoreInfo]:
        cores_retired_instruction_data = KernelSpecificBase.core_retired_instruction(panic_infos)
        cores_state = []

        core_states_raw = panic_infos.pop_until_line_containing('Compressor Info')

        for core_index, core_state_raw in enumerate(core_states_raw):
            try:
                core_retired_instruction = cores_retired_instruction_data[core_index]
            except IndexError:
                core_retired_instruction = None

            core_state = CoreInfo(core_index, core_retired_instruction, core_state_raw)
            cores_state.append(core_state)

            if core_state.crashed and crashed_core_state:
                core_state.update_crashed_state(crashed_core_state, kernel_slide)

        return cores_state

    @staticmethod
    def core_retired_instruction(panic_infos: RawCrashStack) -> List[str]:
        cores_retired_instruction_data: List[str] = []

        while True:
            potential_retired_instruction_data = panic_infos.pop_one()
            if potential_retired_instruction_data.find("TPIDRx_ELy") != -1:
                break

            cores_retired_instruction_data.append(potential_retired_instruction_data)

        return cores_retired_instruction_data

    @property
    def crashed_core(self) -> CoreInfo:
        return list(filter(lambda core_state: core_state.crashed, self.__cores_state))[0]

    @property
    def sliders(self) -> KernelSliders:
        return self.__sliders

    @property
    def versions(self) -> KernelPanicVersions:
        return self.__versions
